

#define _USE_MATH_DEFINES
#include <math.h>
#include <float.h>

#include "Application.h"
#include "Colors.h"

#include <gmtl/gmtl.h>





/*
 *	Populates the scene vector with cubes.
 *
 *	Randomly distributes num cubes inside the box defined by its corners extentLo and extentHi
 *	Each cube is assigned a random orientation and a random color
 *	Cube size is presribed by size
 *
 */

void Application::PopulateScene(const gmtl::Vec3f& extentLo, const gmtl::Vec3f& extentHi, float size, unsigned int num)
{
}





/*
 *	Draws all the cubes stored in the scene vector
 *	Geometry is transformed by GL_MODELVIEW matrix
 */

void Application::DrawScene()
{
	glMatrixMode(GL_MODELVIEW);

	glPushAttrib(GL_ALL_ATTRIB_BITS);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	gmtl::Vec4f lightPosition(1.0f, 1.0f, 1.0f, 0.0f);
	gmtl::Vec4f lightAmbient(0.2f, 0.2f, 0.2f, 0.0f);
	gmtl::Vec4f lightDiffuse(0.5f, 0.5f, 0.5f, 0.0f);
	gmtl::Vec4f lightSpecular(1.0f, 1.0f, 1.0f, 0.0f);

	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition.mData);
	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient.mData);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse.mData);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular.mData);

	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);


	// Cube faces

	glEnable(GL_LIGHTING);
	glPolygonMode(GL_FRONT, GL_FILL);

	glPolygonOffset(1.0, 1.0);
	glEnable(GL_POLYGON_OFFSET_FILL);


	for(unsigned int i = 0 ; i < scene.size() ; i++)
	{
		glMaterialfv(GL_FRONT, GL_AMBIENT, scene[i].color.mData);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, scene[i].color.mData);

		glPushMatrix();
		glMultMatrixf(scene[i].pos.mData);
		glCallList(*cubeDList);
		glPopMatrix();
	}

	glDisable(GL_POLYGON_OFFSET_FILL);


	// Cube outlines

	glLineWidth(1.0f);

	glDisable(GL_LIGHTING);
	glPolygonMode(GL_FRONT, GL_LINE);

	glColor3f(0.6f, 0.6f, 0.6f);		


	for(unsigned int i = 0 ; i < scene.size() ; i++)
	{
		glPushMatrix();
		glMultMatrixf(scene[i].pos.mData);
		glCallList(*cubeDList);		
		glPopMatrix();
	}


	// Highlight selected cube

	if(selectedCube)
	{
		glLineWidth(2.0f);
		glColor3f(1.0, 0.0, 0.0);

		glPushMatrix();
		glMultMatrixf(selectedCube->pos.mData);
		glScalef(1.1f, 1.1f, 1.1f);
		glCallList(*cubeDList);

		glPopMatrix();
	}

	glPopAttrib();
}


