



#ifndef COLORS_H
#define COLORS_H

#include <gmtl/Vec.h>


inline gmtl::Vec3f RGBFromHSV(const gmtl::Vec3f& hsv)
{
	int hInt = ((int)(hsv[0] / 60.0f)) % 6;
	float f = hsv[0] / 60.0f - hInt;

	float p = hsv[2] * (1.0f - hsv[1]);
	float q = hsv[2] * (1.0f - f * hsv[1]);
	float t = hsv[2] * (1.0f - (1.0f - f) * hsv[1]);

	switch(hInt)
	{
	case 0: 
		return gmtl::Vec3f(hsv[2], t, p);

	case 1: 
		return gmtl::Vec3f(q, hsv[2], p);

	case 2: 
		return gmtl::Vec3f(p, hsv[2], t);

	case 3: 
		return gmtl::Vec3f(p, q, hsv[2]);

	case 4: 
		return gmtl::Vec3f(t, p, hsv[2]);

	case 5: 
		return gmtl::Vec3f(hsv[2], p, q);

	default:
		return gmtl::Vec3f(0.0f, 0.0f, 0.0f);
	}
}



inline gmtl::Vec3f HSVFromRGB(const gmtl::Vec3f& rgb)
{
	gmtl::Vec3f hsv;

	float maxVal = rgb[0];
	maxVal = maxVal < rgb[1] ? rgb[1] : maxVal;
	maxVal = maxVal < rgb[2] ? rgb[2] : maxVal;

	float minVal = rgb[0];
	minVal = minVal > rgb[1] ? rgb[1] : minVal;
	minVal = minVal > rgb[2] ? rgb[2] : minVal;


	if(maxVal == minVal)
		hsv[0] = 0.0f;
	else if(maxVal == rgb[0])
		hsv[0] = 60.0f * (rgb[1] - rgb[2]) / (maxVal - minVal) + (rgb[1] >= rgb[2] ? 0.0f : 360.0f);
	else if(maxVal == rgb[1])
		hsv[0] = 60.0f * (rgb[2] - rgb[0]) / (maxVal - minVal) + 120.0f;
	else
		hsv[0] = 60.0f * (rgb[0] - rgb[1]) / (maxVal - minVal) + 240.0f;

	hsv[1] = maxVal == 0 ? 0.0f : (1.0f - minVal / maxVal);
	hsv[2] = maxVal;

	return hsv;
}



inline gmtl::Vec3f HSVInterpolate(const gmtl::Vec3f& src, const gmtl::Vec3f& dst, const float param)
{
	float dP0ToP1 = dst[0] - src[0];
	float dP1ToP0;

	if(dP0ToP1 > 0)
		dP1ToP0 = 360.0f - dP0ToP1;
	else
	{
		dP1ToP0 = - dP0ToP1;
		dP0ToP1 += 360.0f;		
	}


	if(src[0] < dst[0])
	{
		if(dP0ToP1 < dP1ToP0)
			return (1.0f - param) * src + param * dst;
		else
		{
			float h = (1.0f - param) * (src[0] + 360.0f) + param * dst[0];

			if(h >= 360.0f)
				h -= 360.0f;

			return gmtl::Vec3f(h, (1.0f - param) * src[1] + param * dst[1], (1.0f - param) * src[2] + param * dst[2]);
		}
	}
	else
	{
		if(dP0ToP1 < dP1ToP0)
		{
			float h = (1.0f - param) * src[0] + param * (dst[0] + 360.0f);

			if(h >= 360.0f)
				h -= 360.0f;

			return gmtl::Vec3f(h, (1.0f - param) * src[1] + param * dst[1], (1.0f - param) * src[2] + param * dst[2]);
		}
		else
			return (1.0f - param) * src + param * dst;
	}
}


#endif

