
#include <float.h>

#include "Cube.h"



float RayCubeIntersection(const Cube& c, const gmtl::Vec3f& origin, const gmtl::Vec3f& dir)
{
	gmtl::Vec4f o4(origin[0], origin[1], origin[2], 1.0f);
	gmtl::Vec4f d4(dir[0], dir[1], dir[2], 0.0f);

	gmtl::Vec4f lo4 = c.posInvert * o4;
	gmtl::Vec4f ld4 = c.posInvert * d4;

	gmtl::Vec3f localOrigin = gmtl::Vec3f(lo4[0], lo4[1], lo4[2]);
	gmtl::Vec3f localDirection = gmtl::Vec3f(ld4[0], ld4[1], ld4[2]);

	
	float t0, t1;
	float tNear, tFar;

	tNear = -FLT_MAX;
	tFar = FLT_MAX;
	
	
	// ray intersection w/ pair of planes
	
	for(unsigned int i = 0 ; i < 3 ; i++)
		if(localDirection[i])
		{
			t0 = (-0.5f - localOrigin[i]) / localDirection[i];		// First plane lies @ -0.5
			t1 = ( 0.5f - localOrigin[i]) / localDirection[i];		// Scnd plane lies @ +0.5
			
			if(t0 > t1)
			{
				float tmp = t0;
				t0 = t1;
				t1 = tmp;
			}
			
			if(t0 > tNear)
				tNear = t0;
			
			if(t1 < tFar)
				tFar = t1;
			
			if(tNear > tFar || tFar < 0.0f)
				return -1.0f;
		}
		else if(localOrigin[i] < -0.5f || localOrigin[i] > 0.5f)
			return -1.0f;		
	
	return tNear > 0.0f ? tNear : -1.0f;
}

